#! /bin/sh
# aside from this initial boilerplate, this is actually -*- scheme -*- code
main='(module-ref (resolve-module '\''(src xmlt)) '\'main')'
exec ${GUILE-guile} -l $0 -c "(apply $main (cdr (command-line)))" "$@"
!#
;;;; This file is part of XMLTools
;;;; Copyright (C) 2004, 2007 Sergey Poznyakoff
;;;;
;;;; XMLTools is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 3 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; XMLTools is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with XMLTools.  If not, see <http://www.gnu.org/licenses/>.
;;;;

;;; Tailor this statement to your needs if necessary.
;=GUILE_COMMENT=;(set! %load-path (cons "=GUILE_SITE=" %load-path))

;;; Main procedure
(define-module (src xmlt)
  :export (xmlt))

(use-modules (xmltools xmltrans)
	     (xmltools gamlet)
	     (xmltools dict)
	     (ice-9 format)
             (ice-9 rdelim))


;;;; Executable module support
(define (xmlt:error . rest)
  (with-output-to-port
      (current-error-port)
    (lambda ()
      (for-each
       display
       rest)
      (newline))))

(define (usage)
  (display "usage: xmlt [OPTIONS] module [module-options]\n")
  (display "OPTIONS are:\n")
  (display "  --debug=NUMBER    set xmlt debug level\n")
  (display "  --load-path=DIR   append DIR to %load-path\n")
  (display "  -h, --help        display this help summary\n")
  (display "  -V, --version     show program version\n")
  (exit 0))

(define xmlt-module #f)

(define (get-module)
  (if (not xmlt-module)
      (set! xmlt-module (resolve-module '(src xmlt))))
  xmlt-module)
      
(define-macro (bound? name)
  `(and (module-defined? xmlt-module ',name)
	(procedure? ,name)))

(define (xmlt:load arglist)
  (module-use!
   (get-module)
   (resolve-interface (list 'xmltrans (string->symbol (car arglist)))))
  (if (bound? parse-command-line)
      (parse-command-line arglist)
      (cdr arglist)))

(define (xmlt:begin-processing)
  (and (bound? begin-processing)
       (begin-processing)))

(define (xmlt:end-processing)
  (and (bound? end-processing)
       (end-processing))
  (gamlet:flush)
  (exit 0))

(define (version)
  (format #t "~A~%" xmltrans:version-string)
  (display xmltrans:copyright)
  (newline)
  (display xmltrans:license)
  (exit 0))

(define (parse-args args)
  (let loop ((arglist args))
    (if (null? arglist)
	arglist
	(let ((arg (car arglist)))
	  (cond
	   ((or (string=? arg "--help")
		(string=? arg "-h"))
	    (usage))
	   ((or (string=? arg "--debug")
		(string=? arg "-d"))
	    (xmltrans:set-debug-level (string->number (cadr arglist)))
	    (parse-args (cddr arglist)))
	   ((string-prefix? "--debug=" arg)
	    (xmltrans:set-debug-level (string->number (substring arg 8)))
	    (parse-args (cdr arglist)))
	   ((or (string=? arg "--load-path")
		(string=? arg "-L"))
	    (set! %load-path (append
			      (string-split (cadr arglist) #\:)
			      %load-path))
	    (parse-args (cddr arglist)))
	   ((string-prefix? "--load-path=" arg)
	    (set! %load-path (append
			      (string-split (substring arg 12) #\:)
			      %load-path))
	    (parse-args (cdr arglist)))
	   ((or (string=? arg "-V")
		(string=? arg "--version"))
	    (version))
	   ((string=? arg "--")
	    (cdr arglist))
	   (else
	    arglist))))))
				      
(define (xmlt . args)

  ;; Initialize the stuff
  (gamlet:init)
  (gamlet:change-pp-delimiter #\{ #\})

  (let ((arglist (parse-args args)))
    (cond
     ((= 0 (length arglist))
      (xmlt:error "Too few arguments")
      (exit 1))
     (else
      (let ((input-files (xmlt:load arglist)))
	(if (null? input-files)
	    (begin
	      (xmlt:error "Input files not specified")
	      (exit 1)))
	
	(xmlt:begin-processing)

	(for-each
	 (lambda (x)
	   ;; FIXME: in case of exception,
	   ;; gamlet files are not cleaned up
	   (xmltrans:parse-file x))
	 input-files)
	
	(xmlt:end-processing))))))

(define main xmlt)

;;;; xmlt ends here
